"""LLM Providers module for Local Deep Research."""

from .auto_discovery import (
    discover_providers,
    get_discovered_provider_options,
    get_provider_class,
)

__all__ = [
    "discover_providers",
    "get_discovered_provider_options",
    "get_provider_class",
]

# Auto-discover and register all providers on import
discover_providers()
