"""
Local Deep Research - A tool for conducting deep research using AI.
"""

__author__ = "LearningCircuit"
__description__ = "A tool for conducting deep research using AI"

from loguru import logger

from .__version__ import __version__

# Disable logging by default to not interfere with user setup.
logger.disable("local_deep_research")

__all__ = [
    "__version__",
]
