/*---
name = "Light"
label = "Light"
icon = "fa-sun"
group = "core"
type = "light"
description = "Clean and bright for well-lit environments"
---*/
[data-theme="light"] {
  --bg-primary: #f8f9fa;
  --bg-secondary: #ffffff;
  --bg-tertiary: #e9ecef;
  --accent-primary: #5c3dcc;
  --accent-secondary: #7c5ce7;
  --accent-tertiary: #0984e3;
  --text-primary: #212529;
  --text-secondary: #495057;
  --text-muted: #6c757d;
  --border-color: #dee2e6;
  --success-color: #28a745;
  --warning-color: #ffc107;
  --error-color: #dc3545;

  /* RGB variants */
  --bg-primary-rgb: 248, 249, 250;
  --bg-secondary-rgb: 255, 255, 255;
  --bg-tertiary-rgb: 233, 236, 239;
  --accent-primary-rgb: 92, 61, 204;
  --accent-secondary-rgb: 124, 92, 231;
  --accent-tertiary-rgb: 9, 132, 227;
  --text-primary-rgb: 33, 37, 41;
  --text-secondary-rgb: 73, 80, 87;
  --text-muted-rgb: 108, 117, 125;
  --border-color-rgb: 222, 226, 230;
  --success-color-rgb: 40, 167, 69;
  --warning-color-rgb: 255, 193, 7;
  --error-color-rgb: 220, 53, 69;

  /* Shadows and effects */
  --card-shadow: 0 2px 12px rgba(0, 0, 0, 0.08);
  --glow-effect: 0 0 10px rgba(92, 61, 204, 0.15);
  --gradient-bg: linear-gradient(135deg, #e9ecef 0%, #f8f9fa 100%);
  --overlay-color: rgba(0, 0, 0, 0.4);
  --backdrop-blur: blur(10px);

  /* Theme-aware colors for shadows and highlights */
  --shadow-color-rgb: 0, 0, 0;
  --highlight-color-rgb: 0, 0, 0;

  /* Aliases */
  --primary-color: var(--accent-primary);
  --card-bg: var(--bg-secondary);
}

/* Theme-specific adjustments */
[data-theme="light"] ::-webkit-scrollbar-track {
  background: var(--bg-tertiary);
}
[data-theme="light"] ::-webkit-scrollbar-thumb {
  background: var(--accent-primary);
}
